@@ -1,9 +1,11 @@
 # Copyright 2002 by Andrew Dalke.  All rights reserved.
 # Revisions 2007-2016 copyright by Peter Cock.  All rights reserved.
 # Revisions 2008-2009 copyright by Cymon J. Cox.  All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 #
 # Note that BioSQL (including the database schema and scripts) is
 # available and licensed separately.  Please consult www.biosql.org
@@ -40,9 +42,11 @@ class DBSeq(Seq):
         self.start = start
 
     def __len__(self):
+        """Return the length of the sequence."""
         return self._length
 
     def __getitem__(self, index):  # Seq API requirement
+        """Return a subsequence or single letter."""
         # Note since Python 2.0, __getslice__ is deprecated
         # and __getitem__ is used instead.
         # See http://docs.python.org/ref/sequence-methods.html
@@ -59,7 +63,7 @@ class DBSeq(Seq):
                                                      self.start + i,
                                                      self.start + i + 1)
         if not isinstance(index, slice):
-            raise ValueError("Unexpected index type")
+            raise TypeError("Unexpected index type")
 
         # Return the (sub)sequence as another DBSeq or Seq object
         # (see the Seq obect's __getitem__ method)
@@ -103,10 +107,11 @@ class DBSeq(Seq):
             return Seq(full[::index.step], self.alphabet)
 
     def tostring(self):
-        """Returns the full sequence as a python string (DEPRECATED).
+        """Return the full sequence as a python string (DEPRECATED).
 
         You are now encouraged to use str(my_seq) instead of
-        my_seq.tostring()."""
+        my_seq.tostring().
+        """
         import warnings
         warnings.warn("This method is obsolete; please use str(my_seq) "
                       "instead of my_seq.tostring().",
@@ -116,7 +121,7 @@ class DBSeq(Seq):
                                                  self.start + self._length)
 
     def __str__(self):
-        """Returns the full sequence as a python string."""
+        """Return the full sequence as a python string."""
         return self.adaptor.get_subseq_as_string(self.primary_id,
                                                  self.start,
                                                  self.start + self._length)
@@ -124,18 +129,55 @@ class DBSeq(Seq):
     data = property(tostring, doc="Sequence as string (DEPRECATED)")
 
     def toseq(self):
-        """Returns the full sequence as a Seq object."""
+        """Return the full sequence as a Seq object."""
         # Note - the method name copies that of the MutableSeq object
         return Seq(str(self), self.alphabet)
 
     def __add__(self, other):
+        """Add another sequence or string to this sequence.
+
+        The sequence is first converted to a Seq object before the addition.
+        The returned object is a Seq object, not a DBSeq object.
+        """
         # Let the Seq object deal with the alphabet issues etc
         return self.toseq() + other
 
     def __radd__(self, other):
+        """Add another sequence or string to the left.
+
+        The sequence is first converted to a Seq object before the addition.
+        The returned object is a Seq object, not a DBSeq object.
+        """
         # Let the Seq object deal with the alphabet issues etc
         return other + self.toseq()
 
+    def __mul__(self, other):
+        """Multiply sequence by an integer.
+
+        The sequence is first converted to a Seq object before multiplication.
+        The returned object is a Seq object, not a DBSeq object.
+        """
+        # Let the Seq object deal with the alphabet issues etc
+        return self.toseq() * other
+
+    def __rmul__(self, other):
+        """Multiply integer by a sequence.
+
+        The sequence is first converted to a Seq object before multiplication.
+        The returned object is a Seq object, not a DBSeq object.
+        """
+        # Let the Seq object deal with the alphabet issues etc
+        return other * self.toseq()
+
+    def __imul__(self, other):
+        """Multiply sequence by integer in-place.
+
+        The sequence is first converted to a Seq object before multiplication.
+        The returned object is a Seq object, not a DBSeq object.
+        """
+        # Let the Seq object deal with the alphabet issues etc
+        return self.toseq() * other
+
 
 def _retrieve_seq_len(adaptor, primary_id):
     # The database schema ensures there will be only one matching row
@@ -204,7 +246,7 @@ def _retrieve_seq(adaptor, primary_id):
 
 
 def _retrieve_dbxrefs(adaptor, primary_id):
-    """Retrieve the database cross references for the sequence."""
+    """Retrieve the database cross references for the sequence (PRIVATE)."""
     _dbxrefs = []
     dbxrefs = adaptor.execute_and_fetchall(
         "SELECT dbname, accession, version"
@@ -267,12 +309,19 @@ def _retrieve_features(adaptor, primary_id):
             if strand not in (+1, -1, None):
                 raise ValueError("Invalid strand %s found in database for "
                                  "seqfeature_id %s" % (strand, seqfeature_id))
-            if end < start:
+            if start is not None and end is not None and end < start:
                 import warnings
                 from Bio import BiopythonWarning
                 warnings.warn("Inverted location start/end (%i and %i) for "
                               "seqfeature_id %s" % (start, end, seqfeature_id),
                               BiopythonWarning)
+
+            # For SwissProt unknown positions (?)
+            if start is None:
+                start = SeqFeature.UnknownPosition()
+            if end is None:
+                end = SeqFeature.UnknownPosition()
+
             locations.append((location_id, start, end, strand))
         # Get possible remote reference information
         remote_results = adaptor.execute_and_fetchall(
@@ -464,11 +513,8 @@ def _retrieve_taxon(adaptor, primary_id, taxon_id):
             # load_ncbi_taxonomy.pl this is how top parent nodes are stored.
             # Personally, I would have used a NULL parent_taxon_id here.
             break
-        if rank != "no rank":
-            # For consistency with older versions of Biopython, we are only
-            # interested in taxonomy entries with a stated rank.
-            # Add this to the start of the lineage list.
-            taxonomy.insert(0, name)
+
+        taxonomy.insert(0, name)
         taxon_id = parent_taxon_id
 
     if taxonomy:
@@ -493,6 +539,15 @@ class DBSeqRecord(SeqRecord):
     """BioSQL equivalent of the Biopython SeqRecord object."""
 
     def __init__(self, adaptor, primary_id):
+        """Create a DBSeqRecord object.
+
+        Arguments:
+         - adaptor - A BioSQL.BioSeqDatabase.Adaptor object
+         - primary_id - An internal integer ID used by BioSQL
+
+        You wouldn't normally create a DBSeqRecord object yourself,
+        this is done for you when using a BioSeqDatabase object
+        """
         self._adaptor = adaptor
         self._primary_id = primary_id
 
